#ifndef __COPULA_H__
#define __COPULA_H__ 

extern "C"
{
#include "rhelp.h"
}
#include "marginal.h"
#include <fstream>
#include <set>
using namespace std;

#define ZERO_LOGLIKELIHOOD -std::numeric_limits<double>::max()

/*
 * CLASS for the generic implementation of a Bivariate
 * Copula
 */

class Copula
{
 private:

 protected:

  int    mSlice;           //Maximum number of steps for the ``stepping out'' slice sampler
  double wSlice;           //Appropriate window for the ``stepping out'' slice sampler
  double *uSlice, *vSlice; //U data and V data that are used in a particular application of slice sampling
  set<int> *idxSlice;      //Index of data points within U and V vectors that are used in the slice sampler

  virtual void   sliceStepOut(unsigned int which, double x0, double y, double *l, double *r);
  virtual double sliceShrinkage(unsigned int which, double x0, double y, double *l, double *r);
  virtual double evalSlice(unsigned int which, double value) =0;

  virtual double getWSlice(unsigned int which) const;

 public:

  Copula(void);
  virtual ~Copula(void);
  void NewRanks(void);

  virtual void   Cuti(unsigned int n, double *u, double *v) const =0;
  virtual double LLik(unsigned int n, double *u, double *v) const =0;
  virtual double LLik(unsigned int n, double *u, double *v, unsigned int *idx) const =0;
  virtual double LLik(double u, double v) const =0;
  virtual double LLik(double *u, double *v, set<int> *idx) const =0;

  virtual double LLik(Marginal *Mx, Marginal *My) const;
  virtual double LLik(Marginal *Mx, Marginal *My, double x, double y) const;  
  virtual double LLik(Marginal *Mx, Marginal *My, unsigned int d) const;
  virtual double LLik(Marginal *Mx, Marginal *My, unsigned int *idx, unsigned int idx_n) const;
  virtual double LLik(Marginal *Mx, Marginal *My, set<int> *idx) const;

  virtual Copula* RW(unsigned int which, double *lqfwd, double *lqbak) const =0;
  virtual Copula* RW(double start, unsigned int which, double *lqfwd) const =0;
  virtual double  evalRW(unsigned int which, double value) const =0;

  virtual double LPrior(void) const =0;
  virtual void   randomize() = 0;
  virtual double randomizeLL() const =0;

  virtual Copula* sliceSample(unsigned int which, double x0, double *u, double *v, set<int> *idx);
  virtual Copula* sliceSample(unsigned int which, double *u, double *v, set<int> *idx);

  virtual unsigned int NumParams(void) const =0;
  virtual void   Params(double *out) const =0;
  virtual void   setParams(double *in) = 0;
  virtual double getParam(unsigned int which) const =0;

  virtual double spearmanRho() = 0;
  virtual void   getImpliedSpearmanParams(double srho, double out[]) = 0;

  virtual Copula* cloneCopula(unsigned int which, double new_param) const =0;
  virtual double  priorSample() const=0;
};

#endif
