#ifndef __MARGEMPIRICAL_H__
#define __MARGEMPIRICAL_H__ 

#include "marginal.h"
#include <fstream>

/*
 * CLASS for the implementation of a Normal marginal
 * distribution in the context of inferring bivariate
 * distribution defined by a pair of marginals and a copula
 */

class Empirical : public Marginal
{
 private:

  unsigned int numLevels; /* Parameter: number of different levels */
  double       *levels;   /* Levels */
  double       *mass;     /* Probability mass 'mass[i]' associated with anything less than or equal to 'levels[i]' */
  double       bottom, top;

  void setInternalParams(double *params);
  double qempirical(double z) const; /* Quantile function */
  double pempirical(double x) const; /* CDF */

 public:

  Empirical(unsigned int n, double *x, double *d);
  Empirical(double *params, const Empirical *old);

  virtual ~Empirical(void);
  virtual double* NewRanks(unsigned int n, double *z);
  virtual void CalcRanks(void);
  virtual double CalcRanks(double x) const;
  virtual double LLik(void) const;
  virtual double LLik(double x) const;
  virtual unsigned int NumParams(void) const;
  virtual void Params(double *out) const;
  virtual Marginal* RW(unsigned int which, double *lqfwd, double *lqbak) const;
  virtual double RWvalue(double value, double *lqfwd, double *lqbak) const;
  virtual double LPrior(void) const;
  virtual void setParams(double *in);
  virtual void randomize();
};

#endif
