extern "C"
{
#include "matrix.h"
#include "rhelp.h"
}
#include "marginal.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <fstream>
using namespace std;


/*
 * Marginal:
 * 
 * the usual constructor function
 */

Marginal::Marginal(void)
{
  n = 0;
  x = z = NULL;
}


/*
 * Marginal:
 * 
 * the usual constructor function
 */

Marginal::Marginal(unsigned int n, double *x)
{
  this->n = n;
  this->x = x;
  z = NULL;
}


/*
 * ~Marginal:
 * 
 * the usual destructor function 
 */

Marginal::~Marginal(void)
{
  if(z) free(z);
}


/*
 * X:
 *
 * return the samples, x
 */

double* Marginal::X(void) const
{
  return x;
}


/*
 * deleteX:
 *
 * delete the samples, x, & set the x-pointer to NULL
 */

void Marginal::deleteX(void)
{
  free(x);
  x = NULL;
}


/*
 * Ranks:
 *
 * return a pointer to the ranks (or quantiles)
 */

double* Marginal::Ranks(void) const
{
  return z;
}


/*
 * GetRank:
 *
 * return the which-th rank
 */

double Marginal::GetRank(unsigned int which) const
{
  assert(which < n);
  return z[which];
}


/*
 * SetRank:
 *
 * set the which-th rank to be the value what
 */

void Marginal::SetRank(unsigned int which, double what)
{
  assert(which < n);
  assert(what >= 0);
  z[which] = what;
}


/*
 * N:
 *
 * return the number of samples managed under this
 * marginal distribution
 */

unsigned int Marginal::N(void) const
{
  return n;
}
