#ifndef __MARGINAL_H__
#define __MARGINAL_H__ 

extern "C"
{
#include "rhelp.h"
}
#include <fstream>


/*
 * CLASS for the generic implementation of a Marginal
 * distribution in the context of inferring bivariate
 * distribution defined by a pair of marginals and a copula
 */

class Marginal
{
 private:

 protected:

  unsigned int n;      /* number of samples, may be zero */
  double *x;           /* pointer to samples, may be NULL */
  double *z;           /* ranks of the samples based on params */

 public:

  Marginal(void);
  Marginal(unsigned int n, double *x);
  virtual ~Marginal(void);

  double* X(void) const;
  double* Ranks(void) const;
  void SetRank(unsigned int which, double what);
  double GetRank(unsigned int which) const;
  void deleteX(void);
  unsigned int N(void) const;

  virtual double* NewRanks(unsigned int n, double *z)=0;
  virtual double LLik(void) const =0;
  virtual double LLik(double x) const =0;
  virtual void CalcRanks(void)=0;
  virtual double CalcRanks(double x) const =0;
  virtual unsigned int NumParams(void) const =0;
  virtual void Params(double *out) const =0;
  virtual Marginal* RW(unsigned int which, double *lqfwd, double *lqbak) const =0;
  virtual double RWvalue(double value, double *lqfwd, double *lqbak) const =0;
  virtual double LPrior(void) const =0;
  virtual void setParams(double *in) =0;
  virtual void randomize() =0;
};

#endif
